/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

import ClayButton from '@clayui/button';
import {useModal} from '@clayui/modal';
import React, {useCallback, useEffect, useState} from 'react';

import SaveTemplateModal from './SaveTemplateModal';

function SaveTemplate({
	formSaveAsTemplateDataQuerySelector,
	formSaveAsTemplateURL,
	portletNamespace,
}) {
	const [disable, setDisable] = useState(true);
	const [visible, setVisible] = useState(false);
	const {observer, onClose} = useModal({
		onClose: () => setVisible(false),
	});
	const onButtonClick = useCallback(() => {
		setVisible(true);
	}, [setVisible]);

	useEffect(() => {
		const externalInput = document.querySelector(
			`#${portletNamespace}internalClassName`
		);

		if (!externalInput) {
			setDisable(false);

			return;
		}

		function handleExternalInputChange() {
			setDisable(false);
		}

		externalInput.addEventListener('change', handleExternalInputChange);

		return () =>
			externalInput.removeEventListener(
				'change',
				handleExternalInputChange
			);
	}, [portletNamespace]);

	return (
		<span className="mr-3">
			<ClayButton
				disabled={disable}
				displayType="secondary"
				id={`${portletNamespace}saveTemplate`}
				onClick={onButtonClick}
				type="button"
			>
				{Liferay.Language.get('save-as-template')}
			</ClayButton>

			{visible && (
				<SaveTemplateModal
					closeModal={onClose}
					formDataQuerySelector={formSaveAsTemplateDataQuerySelector}
					formSubmitURL={formSaveAsTemplateURL}
					namespace={portletNamespace}
					observer={observer}
				/>
			)}
		</span>
	);
}

export default SaveTemplate;
